# coding: utf-8

"""
    Asana

    This is the interface for interacting with the [Asana Platform](https://developers.asana.com). Our API reference is generated from our [OpenAPI spec] (https://raw.githubusercontent.com/Asana/openapi/master/defs/asana_oas.yaml).  # noqa: E501

    OpenAPI spec version: 1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ProjectBaseCurrentStatusUpdate(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'gid': 'str',
        'resource_type': 'str',
        'title': 'str',
        'resource_subtype': 'str'
    }

    attribute_map = {
        'gid': 'gid',
        'resource_type': 'resource_type',
        'title': 'title',
        'resource_subtype': 'resource_subtype'
    }

    def __init__(self, gid=None, resource_type=None, title=None, resource_subtype=None):  # noqa: E501
        """ProjectBaseCurrentStatusUpdate - a model defined in Swagger"""  # noqa: E501
        self._gid = None
        self._resource_type = None
        self._title = None
        self._resource_subtype = None
        self.discriminator = None
        if gid is not None:
            self.gid = gid
        if resource_type is not None:
            self.resource_type = resource_type
        if title is not None:
            self.title = title
        if resource_subtype is not None:
            self.resource_subtype = resource_subtype

    @property
    def gid(self):
        """Gets the gid of this ProjectBaseCurrentStatusUpdate.  # noqa: E501

        Globally unique identifier of the resource, as a string.  # noqa: E501

        :return: The gid of this ProjectBaseCurrentStatusUpdate.  # noqa: E501
        :rtype: str
        """
        return self._gid

    @gid.setter
    def gid(self, gid):
        """Sets the gid of this ProjectBaseCurrentStatusUpdate.

        Globally unique identifier of the resource, as a string.  # noqa: E501

        :param gid: The gid of this ProjectBaseCurrentStatusUpdate.  # noqa: E501
        :type: str
        """

        self._gid = gid

    @property
    def resource_type(self):
        """Gets the resource_type of this ProjectBaseCurrentStatusUpdate.  # noqa: E501

        The base type of this resource.  # noqa: E501

        :return: The resource_type of this ProjectBaseCurrentStatusUpdate.  # noqa: E501
        :rtype: str
        """
        return self._resource_type

    @resource_type.setter
    def resource_type(self, resource_type):
        """Sets the resource_type of this ProjectBaseCurrentStatusUpdate.

        The base type of this resource.  # noqa: E501

        :param resource_type: The resource_type of this ProjectBaseCurrentStatusUpdate.  # noqa: E501
        :type: str
        """

        self._resource_type = resource_type

    @property
    def title(self):
        """Gets the title of this ProjectBaseCurrentStatusUpdate.  # noqa: E501

        The title of the status update.  # noqa: E501

        :return: The title of this ProjectBaseCurrentStatusUpdate.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this ProjectBaseCurrentStatusUpdate.

        The title of the status update.  # noqa: E501

        :param title: The title of this ProjectBaseCurrentStatusUpdate.  # noqa: E501
        :type: str
        """

        self._title = title

    @property
    def resource_subtype(self):
        """Gets the resource_subtype of this ProjectBaseCurrentStatusUpdate.  # noqa: E501

        The subtype of this resource. Different subtypes retain many of the same fields and behavior, but may render differently in Asana or represent resources with different semantic meaning. The `resource_subtype`s for `status` objects represent the type of their parent.  # noqa: E501

        :return: The resource_subtype of this ProjectBaseCurrentStatusUpdate.  # noqa: E501
        :rtype: str
        """
        return self._resource_subtype

    @resource_subtype.setter
    def resource_subtype(self, resource_subtype):
        """Sets the resource_subtype of this ProjectBaseCurrentStatusUpdate.

        The subtype of this resource. Different subtypes retain many of the same fields and behavior, but may render differently in Asana or represent resources with different semantic meaning. The `resource_subtype`s for `status` objects represent the type of their parent.  # noqa: E501

        :param resource_subtype: The resource_subtype of this ProjectBaseCurrentStatusUpdate.  # noqa: E501
        :type: str
        """
        allowed_values = ["project_status_update", "portfolio_status_update", "goal_status_update"]  # noqa: E501
        if resource_subtype not in allowed_values:
            raise ValueError(
                "Invalid value for `resource_subtype` ({0}), must be one of {1}"  # noqa: E501
                .format(resource_subtype, allowed_values)
            )

        self._resource_subtype = resource_subtype

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ProjectBaseCurrentStatusUpdate, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ProjectBaseCurrentStatusUpdate):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Asana

    This is the interface for interacting with the [Asana Platform](https://developers.asana.com). Our API reference is generated from our [OpenAPI spec] (https://raw.githubusercontent.com/Asana/openapi/master/defs/asana_oas.yaml).  # noqa: E501

    OpenAPI spec version: 1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class AuditLogEventContext(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'context_type': 'str',
        'api_authentication_method': 'str',
        'client_ip_address': 'str',
        'user_agent': 'str',
        'oauth_app_name': 'str'
    }

    attribute_map = {
        'context_type': 'context_type',
        'api_authentication_method': 'api_authentication_method',
        'client_ip_address': 'client_ip_address',
        'user_agent': 'user_agent',
        'oauth_app_name': 'oauth_app_name'
    }

    def __init__(self, context_type=None, api_authentication_method=None, client_ip_address=None, user_agent=None, oauth_app_name=None):  # noqa: E501
        """AuditLogEventContext - a model defined in Swagger"""  # noqa: E501
        self._context_type = None
        self._api_authentication_method = None
        self._client_ip_address = None
        self._user_agent = None
        self._oauth_app_name = None
        self.discriminator = None
        if context_type is not None:
            self.context_type = context_type
        if api_authentication_method is not None:
            self.api_authentication_method = api_authentication_method
        if client_ip_address is not None:
            self.client_ip_address = client_ip_address
        if user_agent is not None:
            self.user_agent = user_agent
        if oauth_app_name is not None:
            self.oauth_app_name = oauth_app_name

    @property
    def context_type(self):
        """Gets the context_type of this AuditLogEventContext.  # noqa: E501

        The type of context. Can be one of `web`, `desktop`, `mobile`, `asana_support`, `asana`, `email`, or `api`.  # noqa: E501

        :return: The context_type of this AuditLogEventContext.  # noqa: E501
        :rtype: str
        """
        return self._context_type

    @context_type.setter
    def context_type(self, context_type):
        """Sets the context_type of this AuditLogEventContext.

        The type of context. Can be one of `web`, `desktop`, `mobile`, `asana_support`, `asana`, `email`, or `api`.  # noqa: E501

        :param context_type: The context_type of this AuditLogEventContext.  # noqa: E501
        :type: str
        """
        allowed_values = ["web", "desktop", "mobile", "asana_support", "asana", "email", "api"]  # noqa: E501
        if context_type not in allowed_values:
            raise ValueError(
                "Invalid value for `context_type` ({0}), must be one of {1}"  # noqa: E501
                .format(context_type, allowed_values)
            )

        self._context_type = context_type

    @property
    def api_authentication_method(self):
        """Gets the api_authentication_method of this AuditLogEventContext.  # noqa: E501

        The authentication method used in the context of an API request. Only present if the `context_type` is `api`. Can be one of `cookie`, `oauth`, `personal_access_token`, or `service_account`.  # noqa: E501

        :return: The api_authentication_method of this AuditLogEventContext.  # noqa: E501
        :rtype: str
        """
        return self._api_authentication_method

    @api_authentication_method.setter
    def api_authentication_method(self, api_authentication_method):
        """Sets the api_authentication_method of this AuditLogEventContext.

        The authentication method used in the context of an API request. Only present if the `context_type` is `api`. Can be one of `cookie`, `oauth`, `personal_access_token`, or `service_account`.  # noqa: E501

        :param api_authentication_method: The api_authentication_method of this AuditLogEventContext.  # noqa: E501
        :type: str
        """
        allowed_values = ["cookie", "oauth", "personal_access_token", "service_account"]  # noqa: E501
        if api_authentication_method not in allowed_values:
            raise ValueError(
                "Invalid value for `api_authentication_method` ({0}), must be one of {1}"  # noqa: E501
                .format(api_authentication_method, allowed_values)
            )

        self._api_authentication_method = api_authentication_method

    @property
    def client_ip_address(self):
        """Gets the client_ip_address of this AuditLogEventContext.  # noqa: E501

        The IP address of the client that initiated the event, if applicable.  # noqa: E501

        :return: The client_ip_address of this AuditLogEventContext.  # noqa: E501
        :rtype: str
        """
        return self._client_ip_address

    @client_ip_address.setter
    def client_ip_address(self, client_ip_address):
        """Sets the client_ip_address of this AuditLogEventContext.

        The IP address of the client that initiated the event, if applicable.  # noqa: E501

        :param client_ip_address: The client_ip_address of this AuditLogEventContext.  # noqa: E501
        :type: str
        """

        self._client_ip_address = client_ip_address

    @property
    def user_agent(self):
        """Gets the user_agent of this AuditLogEventContext.  # noqa: E501

        The user agent of the client that initiated the event, if applicable.  # noqa: E501

        :return: The user_agent of this AuditLogEventContext.  # noqa: E501
        :rtype: str
        """
        return self._user_agent

    @user_agent.setter
    def user_agent(self, user_agent):
        """Sets the user_agent of this AuditLogEventContext.

        The user agent of the client that initiated the event, if applicable.  # noqa: E501

        :param user_agent: The user_agent of this AuditLogEventContext.  # noqa: E501
        :type: str
        """

        self._user_agent = user_agent

    @property
    def oauth_app_name(self):
        """Gets the oauth_app_name of this AuditLogEventContext.  # noqa: E501

        The name of the OAuth App that initiated the event. Only present if the `api_authentication_method` is `oauth`.  # noqa: E501

        :return: The oauth_app_name of this AuditLogEventContext.  # noqa: E501
        :rtype: str
        """
        return self._oauth_app_name

    @oauth_app_name.setter
    def oauth_app_name(self, oauth_app_name):
        """Sets the oauth_app_name of this AuditLogEventContext.

        The name of the OAuth App that initiated the event. Only present if the `api_authentication_method` is `oauth`.  # noqa: E501

        :param oauth_app_name: The oauth_app_name of this AuditLogEventContext.  # noqa: E501
        :type: str
        """

        self._oauth_app_name = oauth_app_name

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(AuditLogEventContext, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AuditLogEventContext):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Asana

    This is the interface for interacting with the [Asana Platform](https://developers.asana.com). Our API reference is generated from our [OpenAPI spec] (https://raw.githubusercontent.com/Asana/openapi/master/defs/asana_oas.yaml).  # noqa: E501

    OpenAPI spec version: 1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ProjectTemplateInstantiateProjectRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'name': 'str',
        'team': 'str',
        'public': 'bool',
        'is_strict': 'bool',
        'requested_dates': 'list[ProjectTemplateInstantiateProjectRequestRequestedDates]',
        'requested_roles': 'list[ProjectTemplateInstantiateProjectRequestRequestedRoles]'
    }

    attribute_map = {
        'name': 'name',
        'team': 'team',
        'public': 'public',
        'is_strict': 'is_strict',
        'requested_dates': 'requested_dates',
        'requested_roles': 'requested_roles'
    }

    def __init__(self, name=None, team=None, public=None, is_strict=None, requested_dates=None, requested_roles=None):  # noqa: E501
        """ProjectTemplateInstantiateProjectRequest - a model defined in Swagger"""  # noqa: E501
        self._name = None
        self._team = None
        self._public = None
        self._is_strict = None
        self._requested_dates = None
        self._requested_roles = None
        self.discriminator = None
        self.name = name
        if team is not None:
            self.team = team
        if public is not None:
            self.public = public
        if is_strict is not None:
            self.is_strict = is_strict
        if requested_dates is not None:
            self.requested_dates = requested_dates
        if requested_roles is not None:
            self.requested_roles = requested_roles

    @property
    def name(self):
        """Gets the name of this ProjectTemplateInstantiateProjectRequest.  # noqa: E501

        The name of the new project.  # noqa: E501

        :return: The name of this ProjectTemplateInstantiateProjectRequest.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ProjectTemplateInstantiateProjectRequest.

        The name of the new project.  # noqa: E501

        :param name: The name of this ProjectTemplateInstantiateProjectRequest.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def team(self):
        """Gets the team of this ProjectTemplateInstantiateProjectRequest.  # noqa: E501

        *Optional*. Sets the team of the new project. If the project template exists in an _organization_, you may specify a value for `team`. If no value is provided then it defaults to the same team as the project template.  # noqa: E501

        :return: The team of this ProjectTemplateInstantiateProjectRequest.  # noqa: E501
        :rtype: str
        """
        return self._team

    @team.setter
    def team(self, team):
        """Sets the team of this ProjectTemplateInstantiateProjectRequest.

        *Optional*. Sets the team of the new project. If the project template exists in an _organization_, you may specify a value for `team`. If no value is provided then it defaults to the same team as the project template.  # noqa: E501

        :param team: The team of this ProjectTemplateInstantiateProjectRequest.  # noqa: E501
        :type: str
        """

        self._team = team

    @property
    def public(self):
        """Gets the public of this ProjectTemplateInstantiateProjectRequest.  # noqa: E501

        Sets the project to public to its team.  # noqa: E501

        :return: The public of this ProjectTemplateInstantiateProjectRequest.  # noqa: E501
        :rtype: bool
        """
        return self._public

    @public.setter
    def public(self, public):
        """Sets the public of this ProjectTemplateInstantiateProjectRequest.

        Sets the project to public to its team.  # noqa: E501

        :param public: The public of this ProjectTemplateInstantiateProjectRequest.  # noqa: E501
        :type: bool
        """

        self._public = public

    @property
    def is_strict(self):
        """Gets the is_strict of this ProjectTemplateInstantiateProjectRequest.  # noqa: E501

        *Optional*. If set to `true`, the endpoint returns an \"Unprocessable Entity\" error if you fail to provide a calendar date value for any date variable. If set to `false`, a default date is used for each unfulfilled date variable (e.g., the current date is used as the Start Date of a project).  # noqa: E501

        :return: The is_strict of this ProjectTemplateInstantiateProjectRequest.  # noqa: E501
        :rtype: bool
        """
        return self._is_strict

    @is_strict.setter
    def is_strict(self, is_strict):
        """Sets the is_strict of this ProjectTemplateInstantiateProjectRequest.

        *Optional*. If set to `true`, the endpoint returns an \"Unprocessable Entity\" error if you fail to provide a calendar date value for any date variable. If set to `false`, a default date is used for each unfulfilled date variable (e.g., the current date is used as the Start Date of a project).  # noqa: E501

        :param is_strict: The is_strict of this ProjectTemplateInstantiateProjectRequest.  # noqa: E501
        :type: bool
        """

        self._is_strict = is_strict

    @property
    def requested_dates(self):
        """Gets the requested_dates of this ProjectTemplateInstantiateProjectRequest.  # noqa: E501

        Array of mappings of date variables to calendar dates.  # noqa: E501

        :return: The requested_dates of this ProjectTemplateInstantiateProjectRequest.  # noqa: E501
        :rtype: list[ProjectTemplateInstantiateProjectRequestRequestedDates]
        """
        return self._requested_dates

    @requested_dates.setter
    def requested_dates(self, requested_dates):
        """Sets the requested_dates of this ProjectTemplateInstantiateProjectRequest.

        Array of mappings of date variables to calendar dates.  # noqa: E501

        :param requested_dates: The requested_dates of this ProjectTemplateInstantiateProjectRequest.  # noqa: E501
        :type: list[ProjectTemplateInstantiateProjectRequestRequestedDates]
        """

        self._requested_dates = requested_dates

    @property
    def requested_roles(self):
        """Gets the requested_roles of this ProjectTemplateInstantiateProjectRequest.  # noqa: E501

        Array of mappings of template roles to user ids  # noqa: E501

        :return: The requested_roles of this ProjectTemplateInstantiateProjectRequest.  # noqa: E501
        :rtype: list[ProjectTemplateInstantiateProjectRequestRequestedRoles]
        """
        return self._requested_roles

    @requested_roles.setter
    def requested_roles(self, requested_roles):
        """Sets the requested_roles of this ProjectTemplateInstantiateProjectRequest.

        Array of mappings of template roles to user ids  # noqa: E501

        :param requested_roles: The requested_roles of this ProjectTemplateInstantiateProjectRequest.  # noqa: E501
        :type: list[ProjectTemplateInstantiateProjectRequestRequestedRoles]
        """

        self._requested_roles = requested_roles

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ProjectTemplateInstantiateProjectRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ProjectTemplateInstantiateProjectRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Asana

    This is the interface for interacting with the [Asana Platform](https://developers.asana.com). Our API reference is generated from our [OpenAPI spec] (https://raw.githubusercontent.com/Asana/openapi/master/defs/asana_oas.yaml).  # noqa: E501

    OpenAPI spec version: 1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import unittest

import asana
from asana.api.sections_api import SectionsApi  # noqa: E501
from asana.rest import ApiException


class TestSectionsApi(unittest.TestCase):
    """SectionsApi unit test stubs"""

    def setUp(self):
        self.api = SectionsApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_add_task_for_section(self):
        """Test case for add_task_for_section

        Add task to section  # noqa: E501
        """
        pass

    def test_create_section_for_project(self):
        """Test case for create_section_for_project

        Create a section in a project  # noqa: E501
        """
        pass

    def test_delete_section(self):
        """Test case for delete_section

        Delete a section  # noqa: E501
        """
        pass

    def test_get_section(self):
        """Test case for get_section

        Get a section  # noqa: E501
        """
        pass

    def test_get_sections_for_project(self):
        """Test case for get_sections_for_project

        Get sections in a project  # noqa: E501
        """
        pass

    def test_insert_section_for_project(self):
        """Test case for insert_section_for_project

        Move or Insert sections  # noqa: E501
        """
        pass

    def test_update_section(self):
        """Test case for update_section

        Update a section  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Asana

    This is the interface for interacting with the [Asana Platform](https://developers.asana.com). Our API reference is generated from our [OpenAPI spec] (https://raw.githubusercontent.com/Asana/openapi/master/defs/asana_oas.yaml).  # noqa: E501

    OpenAPI spec version: 1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class StoryResponseCustomField(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'gid': 'str',
        'resource_type': 'str',
        'name': 'str',
        'resource_subtype': 'str',
        'type': 'str',
        'enum_options': 'list[CustomFieldBaseEnumOptions]',
        'enabled': 'bool',
        'is_formula_field': 'bool',
        'date_value': 'CustomFieldBaseDateValue',
        'enum_value': 'CustomFieldBaseEnumValue',
        'multi_enum_values': 'list[CustomFieldBaseEnumOptions]',
        'number_value': 'float',
        'text_value': 'str',
        'display_value': 'str'
    }

    attribute_map = {
        'gid': 'gid',
        'resource_type': 'resource_type',
        'name': 'name',
        'resource_subtype': 'resource_subtype',
        'type': 'type',
        'enum_options': 'enum_options',
        'enabled': 'enabled',
        'is_formula_field': 'is_formula_field',
        'date_value': 'date_value',
        'enum_value': 'enum_value',
        'multi_enum_values': 'multi_enum_values',
        'number_value': 'number_value',
        'text_value': 'text_value',
        'display_value': 'display_value'
    }

    def __init__(self, gid=None, resource_type=None, name=None, resource_subtype=None, type=None, enum_options=None, enabled=None, is_formula_field=None, date_value=None, enum_value=None, multi_enum_values=None, number_value=None, text_value=None, display_value=None):  # noqa: E501
        """StoryResponseCustomField - a model defined in Swagger"""  # noqa: E501
        self._gid = None
        self._resource_type = None
        self._name = None
        self._resource_subtype = None
        self._type = None
        self._enum_options = None
        self._enabled = None
        self._is_formula_field = None
        self._date_value = None
        self._enum_value = None
        self._multi_enum_values = None
        self._number_value = None
        self._text_value = None
        self._display_value = None
        self.discriminator = None
        if gid is not None:
            self.gid = gid
        if resource_type is not None:
            self.resource_type = resource_type
        if name is not None:
            self.name = name
        if resource_subtype is not None:
            self.resource_subtype = resource_subtype
        if type is not None:
            self.type = type
        if enum_options is not None:
            self.enum_options = enum_options
        if enabled is not None:
            self.enabled = enabled
        if is_formula_field is not None:
            self.is_formula_field = is_formula_field
        if date_value is not None:
            self.date_value = date_value
        if enum_value is not None:
            self.enum_value = enum_value
        if multi_enum_values is not None:
            self.multi_enum_values = multi_enum_values
        if number_value is not None:
            self.number_value = number_value
        if text_value is not None:
            self.text_value = text_value
        if display_value is not None:
            self.display_value = display_value

    @property
    def gid(self):
        """Gets the gid of this StoryResponseCustomField.  # noqa: E501

        Globally unique identifier of the resource, as a string.  # noqa: E501

        :return: The gid of this StoryResponseCustomField.  # noqa: E501
        :rtype: str
        """
        return self._gid

    @gid.setter
    def gid(self, gid):
        """Sets the gid of this StoryResponseCustomField.

        Globally unique identifier of the resource, as a string.  # noqa: E501

        :param gid: The gid of this StoryResponseCustomField.  # noqa: E501
        :type: str
        """

        self._gid = gid

    @property
    def resource_type(self):
        """Gets the resource_type of this StoryResponseCustomField.  # noqa: E501

        The base type of this resource.  # noqa: E501

        :return: The resource_type of this StoryResponseCustomField.  # noqa: E501
        :rtype: str
        """
        return self._resource_type

    @resource_type.setter
    def resource_type(self, resource_type):
        """Sets the resource_type of this StoryResponseCustomField.

        The base type of this resource.  # noqa: E501

        :param resource_type: The resource_type of this StoryResponseCustomField.  # noqa: E501
        :type: str
        """

        self._resource_type = resource_type

    @property
    def name(self):
        """Gets the name of this StoryResponseCustomField.  # noqa: E501

        The name of the custom field.  # noqa: E501

        :return: The name of this StoryResponseCustomField.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this StoryResponseCustomField.

        The name of the custom field.  # noqa: E501

        :param name: The name of this StoryResponseCustomField.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def resource_subtype(self):
        """Gets the resource_subtype of this StoryResponseCustomField.  # noqa: E501

        The type of the custom field. Must be one of the given values.   # noqa: E501

        :return: The resource_subtype of this StoryResponseCustomField.  # noqa: E501
        :rtype: str
        """
        return self._resource_subtype

    @resource_subtype.setter
    def resource_subtype(self, resource_subtype):
        """Sets the resource_subtype of this StoryResponseCustomField.

        The type of the custom field. Must be one of the given values.   # noqa: E501

        :param resource_subtype: The resource_subtype of this StoryResponseCustomField.  # noqa: E501
        :type: str
        """
        allowed_values = ["text", "enum", "multi_enum", "number", "date", "people"]  # noqa: E501
        if resource_subtype not in allowed_values:
            raise ValueError(
                "Invalid value for `resource_subtype` ({0}), must be one of {1}"  # noqa: E501
                .format(resource_subtype, allowed_values)
            )

        self._resource_subtype = resource_subtype

    @property
    def type(self):
        """Gets the type of this StoryResponseCustomField.  # noqa: E501

        *Deprecated: new integrations should prefer the resource_subtype field.* The type of the custom field. Must be one of the given values.   # noqa: E501

        :return: The type of this StoryResponseCustomField.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this StoryResponseCustomField.

        *Deprecated: new integrations should prefer the resource_subtype field.* The type of the custom field. Must be one of the given values.   # noqa: E501

        :param type: The type of this StoryResponseCustomField.  # noqa: E501
        :type: str
        """
        allowed_values = ["text", "enum", "multi_enum", "number", "date", "people"]  # noqa: E501
        if type not in allowed_values:
            raise ValueError(
                "Invalid value for `type` ({0}), must be one of {1}"  # noqa: E501
                .format(type, allowed_values)
            )

        self._type = type

    @property
    def enum_options(self):
        """Gets the enum_options of this StoryResponseCustomField.  # noqa: E501

        *Conditional*. Only relevant for custom fields of type `enum`. This array specifies the possible values which an `enum` custom field can adopt. To modify the enum options, refer to [working with enum options](/reference/createenumoptionforcustomfield).  # noqa: E501

        :return: The enum_options of this StoryResponseCustomField.  # noqa: E501
        :rtype: list[CustomFieldBaseEnumOptions]
        """
        return self._enum_options

    @enum_options.setter
    def enum_options(self, enum_options):
        """Sets the enum_options of this StoryResponseCustomField.

        *Conditional*. Only relevant for custom fields of type `enum`. This array specifies the possible values which an `enum` custom field can adopt. To modify the enum options, refer to [working with enum options](/reference/createenumoptionforcustomfield).  # noqa: E501

        :param enum_options: The enum_options of this StoryResponseCustomField.  # noqa: E501
        :type: list[CustomFieldBaseEnumOptions]
        """

        self._enum_options = enum_options

    @property
    def enabled(self):
        """Gets the enabled of this StoryResponseCustomField.  # noqa: E501

        *Conditional*. Determines if the custom field is enabled or not.  # noqa: E501

        :return: The enabled of this StoryResponseCustomField.  # noqa: E501
        :rtype: bool
        """
        return self._enabled

    @enabled.setter
    def enabled(self, enabled):
        """Sets the enabled of this StoryResponseCustomField.

        *Conditional*. Determines if the custom field is enabled or not.  # noqa: E501

        :param enabled: The enabled of this StoryResponseCustomField.  # noqa: E501
        :type: bool
        """

        self._enabled = enabled

    @property
    def is_formula_field(self):
        """Gets the is_formula_field of this StoryResponseCustomField.  # noqa: E501

        *Conditional*. This flag describes whether a custom field is a formula custom field.  # noqa: E501

        :return: The is_formula_field of this StoryResponseCustomField.  # noqa: E501
        :rtype: bool
        """
        return self._is_formula_field

    @is_formula_field.setter
    def is_formula_field(self, is_formula_field):
        """Sets the is_formula_field of this StoryResponseCustomField.

        *Conditional*. This flag describes whether a custom field is a formula custom field.  # noqa: E501

        :param is_formula_field: The is_formula_field of this StoryResponseCustomField.  # noqa: E501
        :type: bool
        """

        self._is_formula_field = is_formula_field

    @property
    def date_value(self):
        """Gets the date_value of this StoryResponseCustomField.  # noqa: E501


        :return: The date_value of this StoryResponseCustomField.  # noqa: E501
        :rtype: CustomFieldBaseDateValue
        """
        return self._date_value

    @date_value.setter
    def date_value(self, date_value):
        """Sets the date_value of this StoryResponseCustomField.


        :param date_value: The date_value of this StoryResponseCustomField.  # noqa: E501
        :type: CustomFieldBaseDateValue
        """

        self._date_value = date_value

    @property
    def enum_value(self):
        """Gets the enum_value of this StoryResponseCustomField.  # noqa: E501


        :return: The enum_value of this StoryResponseCustomField.  # noqa: E501
        :rtype: CustomFieldBaseEnumValue
        """
        return self._enum_value

    @enum_value.setter
    def enum_value(self, enum_value):
        """Sets the enum_value of this StoryResponseCustomField.


        :param enum_value: The enum_value of this StoryResponseCustomField.  # noqa: E501
        :type: CustomFieldBaseEnumValue
        """

        self._enum_value = enum_value

    @property
    def multi_enum_values(self):
        """Gets the multi_enum_values of this StoryResponseCustomField.  # noqa: E501

        *Conditional*. Only relevant for custom fields of type `multi_enum`. This object is the chosen values of a `multi_enum` custom field.  # noqa: E501

        :return: The multi_enum_values of this StoryResponseCustomField.  # noqa: E501
        :rtype: list[CustomFieldBaseEnumOptions]
        """
        return self._multi_enum_values

    @multi_enum_values.setter
    def multi_enum_values(self, multi_enum_values):
        """Sets the multi_enum_values of this StoryResponseCustomField.

        *Conditional*. Only relevant for custom fields of type `multi_enum`. This object is the chosen values of a `multi_enum` custom field.  # noqa: E501

        :param multi_enum_values: The multi_enum_values of this StoryResponseCustomField.  # noqa: E501
        :type: list[CustomFieldBaseEnumOptions]
        """

        self._multi_enum_values = multi_enum_values

    @property
    def number_value(self):
        """Gets the number_value of this StoryResponseCustomField.  # noqa: E501

        *Conditional*. This number is the value of a `number` custom field.  # noqa: E501

        :return: The number_value of this StoryResponseCustomField.  # noqa: E501
        :rtype: float
        """
        return self._number_value

    @number_value.setter
    def number_value(self, number_value):
        """Sets the number_value of this StoryResponseCustomField.

        *Conditional*. This number is the value of a `number` custom field.  # noqa: E501

        :param number_value: The number_value of this StoryResponseCustomField.  # noqa: E501
        :type: float
        """

        self._number_value = number_value

    @property
    def text_value(self):
        """Gets the text_value of this StoryResponseCustomField.  # noqa: E501

        *Conditional*. This string is the value of a `text` custom field.  # noqa: E501

        :return: The text_value of this StoryResponseCustomField.  # noqa: E501
        :rtype: str
        """
        return self._text_value

    @text_value.setter
    def text_value(self, text_value):
        """Sets the text_value of this StoryResponseCustomField.

        *Conditional*. This string is the value of a `text` custom field.  # noqa: E501

        :param text_value: The text_value of this StoryResponseCustomField.  # noqa: E501
        :type: str
        """

        self._text_value = text_value

    @property
    def display_value(self):
        """Gets the display_value of this StoryResponseCustomField.  # noqa: E501

        A string representation for the value of the custom field. Integrations that don't require the underlying type should use this field to read values. Using this field will future-proof an app against new custom field types.  # noqa: E501

        :return: The display_value of this StoryResponseCustomField.  # noqa: E501
        :rtype: str
        """
        return self._display_value

    @display_value.setter
    def display_value(self, display_value):
        """Sets the display_value of this StoryResponseCustomField.

        A string representation for the value of the custom field. Integrations that don't require the underlying type should use this field to read values. Using this field will future-proof an app against new custom field types.  # noqa: E501

        :param display_value: The display_value of this StoryResponseCustomField.  # noqa: E501
        :type: str
        """

        self._display_value = display_value

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(StoryResponseCustomField, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, StoryResponseCustomField):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Asana

    This is the interface for interacting with the [Asana Platform](https://developers.asana.com). Our API reference is generated from our [OpenAPI spec] (https://raw.githubusercontent.com/Asana/openapi/master/defs/asana_oas.yaml).  # noqa: E501

    OpenAPI spec version: 1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from asana.api_client import ApiClient


class GoalRelationshipsApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def add_supporting_relationship(self, body, goal_gid, **kwargs):  # noqa: E501
        """Add a supporting goal relationship  # noqa: E501

        Creates a goal relationship by adding a supporting resource to a given goal.  Returns the newly created goal relationship record.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_supporting_relationship(body, goal_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param GoalGidAddSupportingRelationshipBody body: The supporting resource to be added to the goal (required)
        :param str goal_gid: Globally unique identifier for the goal. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: GoalRelationshipResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.add_supporting_relationship_with_http_info(body, goal_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.add_supporting_relationship_with_http_info(body, goal_gid, **kwargs)  # noqa: E501
            return data

    def add_supporting_relationship_with_http_info(self, body, goal_gid, **kwargs):  # noqa: E501
        """Add a supporting goal relationship  # noqa: E501

        Creates a goal relationship by adding a supporting resource to a given goal.  Returns the newly created goal relationship record.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_supporting_relationship_with_http_info(body, goal_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param GoalGidAddSupportingRelationshipBody body: The supporting resource to be added to the goal (required)
        :param str goal_gid: Globally unique identifier for the goal. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: GoalRelationshipResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'goal_gid', 'opt_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_supporting_relationship" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `add_supporting_relationship`")  # noqa: E501
        # verify the required parameter 'goal_gid' is set
        if ('goal_gid' not in params or
                params['goal_gid'] is None):
            raise ValueError("Missing the required parameter `goal_gid` when calling `add_supporting_relationship`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'goal_gid' in params:
            path_params['goal_gid'] = params['goal_gid']  # noqa: E501

        query_params = []
        if 'opt_fields' in params:
            query_params.append(('opt_fields', params['opt_fields']))  # noqa: E501
            collection_formats['opt_fields'] = 'csv'  # noqa: E501

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/goals/{goal_gid}/addSupportingRelationship', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='GoalRelationshipResponseData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_goal_relationship(self, goal_relationship_gid, **kwargs):  # noqa: E501
        """Get a goal relationship  # noqa: E501

        Returns the complete updated goal relationship record for a single goal relationship.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_goal_relationship(goal_relationship_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str goal_relationship_gid: Globally unique identifier for the goal relationship. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: GoalRelationshipResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.get_goal_relationship_with_http_info(goal_relationship_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.get_goal_relationship_with_http_info(goal_relationship_gid, **kwargs)  # noqa: E501
            return data

    def get_goal_relationship_with_http_info(self, goal_relationship_gid, **kwargs):  # noqa: E501
        """Get a goal relationship  # noqa: E501

        Returns the complete updated goal relationship record for a single goal relationship.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_goal_relationship_with_http_info(goal_relationship_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str goal_relationship_gid: Globally unique identifier for the goal relationship. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: GoalRelationshipResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['goal_relationship_gid', 'opt_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_goal_relationship" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'goal_relationship_gid' is set
        if ('goal_relationship_gid' not in params or
                params['goal_relationship_gid'] is None):
            raise ValueError("Missing the required parameter `goal_relationship_gid` when calling `get_goal_relationship`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'goal_relationship_gid' in params:
            path_params['goal_relationship_gid'] = params['goal_relationship_gid']  # noqa: E501

        query_params = []
        if 'opt_fields' in params:
            query_params.append(('opt_fields', params['opt_fields']))  # noqa: E501
            collection_formats['opt_fields'] = 'csv'  # noqa: E501

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/goal_relationships/{goal_relationship_gid}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='GoalRelationshipResponseData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_goal_relationships(self, supported_goal, **kwargs):  # noqa: E501
        """Get goal relationships  # noqa: E501

        Returns compact goal relationship records.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_goal_relationships(supported_goal, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str supported_goal: Globally unique identifier for the supported goal in the goal relationship. (required)
        :param int limit: Results per page. The number of objects to return per page. The value must be between 1 and 100.
        :param str offset: Offset token. An offset to the next page returned by the API. A pagination request will return an offset token, which can be used as an input parameter to the next request. If an offset is not passed in, the API will return the first page of results. 'Note: You can only pass in an offset that was returned to you via a previously paginated request.'
        :param str resource_subtype: If provided, filter to goal relationships with a given resource_subtype.
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: GoalRelationshipResponseArray
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.get_goal_relationships_with_http_info(supported_goal, **kwargs)  # noqa: E501
        else:
            (data) = self.get_goal_relationships_with_http_info(supported_goal, **kwargs)  # noqa: E501
            return data

    def get_goal_relationships_with_http_info(self, supported_goal, **kwargs):  # noqa: E501
        """Get goal relationships  # noqa: E501

        Returns compact goal relationship records.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_goal_relationships_with_http_info(supported_goal, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str supported_goal: Globally unique identifier for the supported goal in the goal relationship. (required)
        :param int limit: Results per page. The number of objects to return per page. The value must be between 1 and 100.
        :param str offset: Offset token. An offset to the next page returned by the API. A pagination request will return an offset token, which can be used as an input parameter to the next request. If an offset is not passed in, the API will return the first page of results. 'Note: You can only pass in an offset that was returned to you via a previously paginated request.'
        :param str resource_subtype: If provided, filter to goal relationships with a given resource_subtype.
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: GoalRelationshipResponseArray
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['supported_goal', 'limit', 'offset', 'resource_subtype', 'opt_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_goal_relationships" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'supported_goal' is set
        if ('supported_goal' not in params or
                params['supported_goal'] is None):
            raise ValueError("Missing the required parameter `supported_goal` when calling `get_goal_relationships`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'limit' in params:
            query_params.append(('limit', params['limit']))  # noqa: E501
        if 'offset' in params:
            query_params.append(('offset', params['offset']))  # noqa: E501
        if 'supported_goal' in params:
            query_params.append(('supported_goal', params['supported_goal']))  # noqa: E501
        if 'resource_subtype' in params:
            query_params.append(('resource_subtype', params['resource_subtype']))  # noqa: E501
        if 'opt_fields' in params:
            query_params.append(('opt_fields', params['opt_fields']))  # noqa: E501
            collection_formats['opt_fields'] = 'csv'  # noqa: E501

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/goal_relationships', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='GoalRelationshipResponseArray',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def remove_supporting_relationship(self, body, goal_gid, **kwargs):  # noqa: E501
        """Removes a supporting goal relationship  # noqa: E501

        Removes a goal relationship for a given parent goal.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.remove_supporting_relationship(body, goal_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param GoalGidRemoveSupportingRelationshipBody body: The supporting resource to be removed from the goal (required)
        :param str goal_gid: Globally unique identifier for the goal. (required)
        :return: EmptyResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.remove_supporting_relationship_with_http_info(body, goal_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.remove_supporting_relationship_with_http_info(body, goal_gid, **kwargs)  # noqa: E501
            return data

    def remove_supporting_relationship_with_http_info(self, body, goal_gid, **kwargs):  # noqa: E501
        """Removes a supporting goal relationship  # noqa: E501

        Removes a goal relationship for a given parent goal.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.remove_supporting_relationship_with_http_info(body, goal_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param GoalGidRemoveSupportingRelationshipBody body: The supporting resource to be removed from the goal (required)
        :param str goal_gid: Globally unique identifier for the goal. (required)
        :return: EmptyResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'goal_gid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method remove_supporting_relationship" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `remove_supporting_relationship`")  # noqa: E501
        # verify the required parameter 'goal_gid' is set
        if ('goal_gid' not in params or
                params['goal_gid'] is None):
            raise ValueError("Missing the required parameter `goal_gid` when calling `remove_supporting_relationship`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'goal_gid' in params:
            path_params['goal_gid'] = params['goal_gid']  # noqa: E501

        query_params = []

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/goals/{goal_gid}/removeSupportingRelationship', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='EmptyResponseData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def update_goal_relationship(self, body, goal_relationship_gid, **kwargs):  # noqa: E501
        """Update a goal relationship  # noqa: E501

        An existing goal relationship can be updated by making a PUT request on the URL for that goal relationship. Only the fields provided in the `data` block will be updated; any unspecified fields will remain unchanged.  Returns the complete updated goal relationship record.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.update_goal_relationship(body, goal_relationship_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param GoalRelationshipsGoalRelationshipGidBody body: The updated fields for the goal relationship. (required)
        :param str goal_relationship_gid: Globally unique identifier for the goal relationship. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: GoalRelationshipResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = kwargs.get("_return_http_data_only", True)
        if kwargs.get('async_req'):
            return self.update_goal_relationship_with_http_info(body, goal_relationship_gid, **kwargs)  # noqa: E501
        else:
            (data) = self.update_goal_relationship_with_http_info(body, goal_relationship_gid, **kwargs)  # noqa: E501
            return data

    def update_goal_relationship_with_http_info(self, body, goal_relationship_gid, **kwargs):  # noqa: E501
        """Update a goal relationship  # noqa: E501

        An existing goal relationship can be updated by making a PUT request on the URL for that goal relationship. Only the fields provided in the `data` block will be updated; any unspecified fields will remain unchanged.  Returns the complete updated goal relationship record.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.update_goal_relationship_with_http_info(body, goal_relationship_gid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param GoalRelationshipsGoalRelationshipGidBody body: The updated fields for the goal relationship. (required)
        :param str goal_relationship_gid: Globally unique identifier for the goal relationship. (required)
        :param list[str] opt_fields: This endpoint returns a compact resource, which excludes some properties by default. To include those optional properties, set this query parameter to a comma-separated list of the properties you wish to include.
        :return: GoalRelationshipResponseData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'goal_relationship_gid', 'opt_fields']  # noqa: E501
        all_params.append('async_req')
        all_params.append('header_params')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method update_goal_relationship" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `update_goal_relationship`")  # noqa: E501
        # verify the required parameter 'goal_relationship_gid' is set
        if ('goal_relationship_gid' not in params or
                params['goal_relationship_gid'] is None):
            raise ValueError("Missing the required parameter `goal_relationship_gid` when calling `update_goal_relationship`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'goal_relationship_gid' in params:
            path_params['goal_relationship_gid'] = params['goal_relationship_gid']  # noqa: E501

        query_params = []
        if 'opt_fields' in params:
            query_params.append(('opt_fields', params['opt_fields']))  # noqa: E501
            collection_formats['opt_fields'] = 'csv'  # noqa: E501

        header_params = kwargs.get("header_params", {})

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json; charset=UTF-8'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json; charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['oauth2']  # noqa: E501

        return self.api_client.call_api(
            '/goal_relationships/{goal_relationship_gid}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='GoalRelationshipResponseData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)
